//==========================================================================
//						First Person Experience for Orbiter
//							Created by Escpetomsfate
//	Thanks to T.Neo for the original idea and Artlav for the vector maths
//						main.cpp - Callback functions and entry points
//==========================================================================

//Defines, so the module works properly.
#define STRICT 1
#define ORBITER_MODULE

//includes
#include "windows.h"
#include "orbitersdk.h"

//Global variables
HWND window;
bool move = true;
bool keydown = false;
bool homedown = false;
double camera_velocity = 1.5;
VESSEL* focus;
VECTOR3 v3, camdir, pos, tr;
HINSTANCE hModule;

DLLCLBK void InitModule(HINSTANCE hinstance)
{
	hModule = hinstance;
}

DLLCLBK void opcOpenRenderViewport (HWND renderWnd,DWORD width,DWORD height,BOOL fullscreen)
{
//Set window
window = renderWnd;
//Read in the file
FILEHANDLE config = oapiOpenFile("firstperson.cfg", FILE_IN, CONFIG);
oapiReadItem_float(config, "CAMERAVELOCITY", camera_velocity);
oapiCloseFile(config, FILE_IN);
}

DLLCLBK void opcFocusChanged (OBJHANDLE new_focus,OBJHANDLE old_focus)
{
	//Set focus to focus vessel.
	focus = oapiGetVesselInterface(new_focus);
	//Set rotation range to full 360 range
	focus->SetCameraRotationRange(RAD* 360, RAD* 360, RAD* 360, RAD* 360);
}

DLLCLBK void opcPreStep (double SimT,double SimDT,double mjd)
{
//Only start doing things if we're in VC. No point eating up the FPS. Also, check that the window is in focus, since the keypresses are read globally.
if(oapiCockpitMode () == COCKPIT_VIRTUAL && GetFocus() == window)
{

//Look for the toggle key
if(GetAsyncKeyState(VK_END))
{
	if(keydown == false)
	{
	keydown = true;
	//reset move
	move = !move;
	}
} else {keydown = false;}

//look for mesh key
if(GetAsyncKeyState(VK_F5) && homedown == false)
{
	//iterate through 100 meshes TODO - fix!
	for(int i = 0; i < 100; i++)
	{
	//Set the mesh vis to always...
	focus->SetMeshVisibilityMode(i, MESHVIS_ALWAYS);
	}
	homedown = true;
} else homedown = false;

//continue if toggle is on
if(move == true)
{
	//Get camera offset from vessel centre
	focus->GetCameraOffset(v3);
	//Get camera direction (in global co-ords)
	oapiCameraGlobalDir(&camdir);
	//Get global position of vessel so we can translate
	focus->GetGlobalPos(pos);
	//Translate from global to local co-ordinates. Thanks to Artlav for helping with this...
	focus->Global2Local(pos + camdir, tr);

	//check for keypress of W
if(GetAsyncKeyState(87))
{
	//move forward
	v3 +=  (tr * camera_velocity)*SimDT;
}

//Check for keypress of S
if(GetAsyncKeyState(83))
{
	//move backward
	v3 -=  (tr * camera_velocity)*SimDT;
}

//Apply new position
oapiGetFocusInterface()->SetCameraOffset(v3);
focus->SetCameraRotationRange(RAD* 360, RAD* 360, RAD* 360, RAD* 360);
}
}
}

DLLCLBK void ExitModule(HINSTANCE hinstance){}